puts "=================== starting Android Lint Entropy Reducer ===================="

# ========================  SETUP ============================

# User name for git commits made by this script.
TRAVIS_GIT_USERNAME = String.new("Travis CI server")

# File name and relative path of generated Lint report. Must match build.gradle file:
#   lintOptions {
#       htmlOutput file("[FILE_NAME].html")
#   }
LINT_REPORT_FILE = String.new("8vim/lint-report.html")

# File name and relative path of previous results of this script.
PREVIOUS_LINT_RESULTS_FILE=String.new("lint-report/lint-results.txt")

# Flag to evaluate warnings. true = check warnings; false = ignore warnings
CHECK_WARNINGS = true

# File name and relative path to custom lint rules; Can be null or "".
CUSTOM_LINT_FILE = String.new("")

# ================ SETUP DONE; DON'T TOUCH ANYTHING BELOW  ================

require 'fileutils'
require 'pathname'
require 'open3'

# since we need the xml-simple gem, and we want this script self-contained, let's grab it just when we need it
begin
    gem "xml-simple"
    rescue LoadError
    system("gem install xml-simple")
    Gem.clear_paths
end

require 'xmlsimple'

# add custom Lint jar
if CUSTOM_LINT_FILE != nil &&
    CUSTOM_LINT_FILE.length > 0

    ENV["ANDROID_LINT_JARS"] = Dir.pwd + "/" + CUSTOM_LINT_FILE
    puts "adding custom lint rules to default set: "
    puts ENV["ANDROID_LINT_JARS"]
end

# run Lint
puts "running Lint..."
#system './gradlew clean lint'
output = `./gradlew clean lint`

# confirm that Lint ran w/out error
result = $?.to_i
if result != 0
    puts "FAIL: failed to run ./gradlew clean lint"
    exit 1
end

issues_string = output[/[0-9]+ issues found/]

# find number of errors
current_issue_count = issues_string.match(/[0-9]*/)[0].to_i
puts "found issues: " + current_issue_count.to_s

# get previous error and warning counts from last successful build

previous_results = false

previous_lint_reports = Dir.glob(PREVIOUS_LINT_RESULTS_FILE)
if previous_lint_reports == nil ||
    previous_lint_reports.length == 0

    previous_lint_report = File.new(PREVIOUS_LINT_RESULTS_FILE, "w") # create for writing to later
else
    previous_lint_report = String.new(previous_lint_reports[0])

    previous_error_warning_string = ""
    File.open previous_lint_report do |file|
      previous_error_warning_string = file.find { |line| line =~ /[0-9]+ issues found/ }
    end

    unless previous_error_warning_string.nil?
        previous_results = true

        previous_issue_count = previous_error_warning_string[/[0-9]+/].to_i
        puts "previous issue count: " + previous_issue_count.to_s
    end
end

# compare previous error count with current error count
if previous_results == true  &&
    current_issue_count > previous_issue_count
    puts ("FAIL: issue count increased from #{previous_issue_count} to #{current_issue_count}")
    puts "Please run './gradew clean lint' locally and fix lint issues"
    exit 1
end

# check if warning and error count stayed the same
if  previous_results == true &&
    current_issue_count == previous_issue_count

    puts "SUCCESS: count stayed the same"
    exit 0
end

# issue count DECREASED

# write new results to file (will overwrite existing, or create new)
File.write(previous_lint_report, "DO NOT TOUCH; GENERATED BY TRAVIS\n" + issues_string )

# push changes to github (if this script is run locally, we don't want to overwrite git username and email, so save temporarily)
previous_git_username, st = Open3.capture2('git config user.name')
previous_git_username = previous_git_username.strip

previous_git_email, st = Open3.capture3('git config user.email')
previous_git_email = previous_git_email.strip

# update git user name and email for this script
system ("git config --local user.name '"  + TRAVIS_GIT_USERNAME + "'")
system ("git config --local user.email '.'") # set email blank

# add previous Lint result file to git
system ('git add ' + PREVIOUS_LINT_RESULTS_FILE)

# Travis has git in a detached head here. Let's get on the right branch.
travis_branch_name = ENV['TRAVIS_BRANCH']
if travis_branch_name != nil
    system ('git checkout ' + travis_branch_name)
end

# commit changes; Add "skip ci" so that we don't accidentally trigger another Travis build
system ('git commit -m "Travis: update Lint results to reflect reduced error/warning count [skip ci]" ')

# push to origin
system("git config push.default simple")
system ("git push")

# restore previous git user name and email
system("git config --local user.name '#{previous_git_username}'")
system("git config --local user.email '#{previous_git_email}'")

puts "SUCCESS: count was reduced"
exit 0 # success
